@@ -3,10 +3,10 @@
 Custom Datastructures
 
 """
-from UserList import UserList
-from Queue import Queue
-from Queue import Empty as QueueEmpty
+import time
 import traceback
+from UserList import UserList
+from Queue import Queue, Empty as QueueEmpty
 
 
 class PositionQueue(UserList):
@@ -27,6 +27,8 @@ class PositionQueue(UserList):
         """Describes an unfilled slot."""
 
         def __init__(self, position):
+            # This is not used, but is an argument from xrange
+            # so why not.
             self.position = position
 
     def __init__(self, length):
@@ -54,7 +56,6 @@ class ExceptionInfo(object):
     :param exc_info: The exception tuple info as returned by
         :func:`traceback.format_exception`.
 
-
     .. attribute:: exception
 
         The original exception.
@@ -68,20 +69,35 @@ class ExceptionInfo(object):
     def __init__(self, exc_info):
         type_, exception, tb = exc_info
         self.exception = exception
-        self.traceback = '\n'.join(traceback.format_exception(*exc_info))
+        self.traceback = ''.join(traceback.format_exception(*exc_info))
 
     def __str__(self):
-        return str(self.exception)
+        return self.traceback
 
     def __repr__(self):
-        return "<%s.%s: %s" % (
+        return "<%s.%s: %s>" % (
                 self.__class__.__module__,
                 self.__class__.__name__,
                 str(self.exception))
 
 
 def consume_queue(queue):
-    while True:
+    """Iterator yielding all immediately available items in a
+    :class:`Queue.Queue`.
+
+    The iterator stops as soon as the queue raises :exc:`Queue.Empty`.
+
+    Example
+
+        >>> q = Queue()
+        >>> map(q.put, range(4))
+        >>> list(consume_queue(q))
+        [0, 1, 2, 3]
+        >>> list(consume_queue(q))
+        []
+
+    """
+    while 1:
         try:
             yield queue.get_nowait()
         except QueueEmpty:
@@ -89,7 +105,7 @@ def consume_queue(queue):
 
 
 class SharedCounter(object):
-    """An integer that can be updated by several threads at once.
+    """Thread-safe counter.
 
     Please note that the final value is not synchronized, this means
     that you should not update the value by using a previous value, the only
@@ -146,3 +162,64 @@ class SharedCounter(object):
 
     def __repr__(self):
         return "<SharedCounter: int(%s)>" % str(int(self))
+
+
+class LimitedSet(object):
+    """Kind-of Set with limitations.
+
+    Good for when you need to test for membership (``a in set``),
+    but the list might become to big, so you want to limit it so it doesn't
+    consume too much resources.
+
+    :keyword maxlen: Maximum number of members before we start
+        deleting expired members.
+    :keyword expires: Time in seconds, before a membership expires.
+
+    """
+
+    def __init__(self, maxlen=None, expires=None):
+        self.maxlen = maxlen
+        self.expires = expires
+        self._data = {}
+
+    def add(self, value):
+        """Add a new member."""
+        self._expire_item()
+        self._data[value] = time.time()
+
+    def pop_value(self, value):
+        """Remove membership by finding value."""
+        self._data.pop(value, None)
+
+    def _expire_item(self):
+        """Hunt down and remove an expired item."""
+        while 1:
+            if self.maxlen and len(self) >= self.maxlen:
+                value, when = self.first
+                if not self.expires or time.time() > when + self.expires:
+                    try:
+                        self.pop_value(value)
+                    except TypeError: # pragma: no cover
+                        continue
+            break
+
+    def __contains__(self, value):
+        return value in self._data
+
+    def __iter__(self):
+        return iter(self._data.keys())
+
+    def __len__(self):
+        return len(self._data.keys())
+
+    def __repr__(self):
+        return "LimitedSet([%s])" % (repr(self._data.keys()))
+
+    @property
+    def chronologically(self):
+        return sorted(self._data.items(), key=lambda (value, when): when)
+
+    @property
+    def first(self):
+        """Get the oldest member."""
+        return self.chronologically[0]
