@@ -1,4 +1,28 @@
-"""celery.exceptions"""
+"""
+
+Common Exceptions
+
+"""
+
+UNREGISTERED_FMT = """
+Task of kind %s is not registered, please make sure it's imported.
+""".strip()
+
+
+class NotRegistered(KeyError):
+    """The task is not registered."""
+
+    def __init__(self, message, *args, **kwargs):
+        message = UNREGISTERED_FMT % str(message)
+        super(NotRegistered, self).__init__(message, *args, **kwargs)
+
+
+class AlreadyRegistered(Exception):
+    """The task is already registered."""
+
+
+class TimeoutError(Exception):
+    """The operation timed out."""
 
 
 class MaxRetriesExceededError(Exception):
@@ -11,15 +35,3 @@ class RetryTaskError(Exception):
     def __init__(self, message, exc, *args, **kwargs):
         self.exc = exc
         super(RetryTaskError, self).__init__(message, exc, *args, **kwargs)
-
-
-class NotRegistered(Exception):
-    """The task is not registered."""
-
-
-class AlreadyRegistered(Exception):
-    """The task is already registered."""
-
-
-class TimeoutError(Exception):
-    """The operation timed out."""
