@@ -1,13 +1,16 @@
 import sys
-import unittest
 import errno
 import socket
-from celery.backends import tyrant
-from celery.backends.tyrant import Backend as TyrantBackend
-from celery.utils import gen_unique_id
+import unittest
+
 from django.core.exceptions import ImproperlyConfigured
 
-_no_tyrant_msg = "* Tokyo Tyrant not running. Will not execute related tests."
+from celery import states
+from celery.utils import gen_unique_id
+from celery.backends import tyrant
+from celery.backends.tyrant import TyrantBackend
+
+_no_tyrant_msg = "* Tokyo Tyrant %s. Will not execute related tests."
 _no_tyrant_msg_emitted = False
 
 
@@ -19,28 +22,28 @@ class SomeClass(object):
 
 def get_tyrant_or_None():
 
-    def emit_no_tyrant_msg():
+    def emit_no_tyrant_msg(reason):
         global _no_tyrant_msg_emitted
         if not _no_tyrant_msg_emitted:
-            sys.stderr.write("\n" + _no_tyrant_msg + "\n")
+            sys.stderr.write("\n" + _no_tyrant_msg % reason + "\n")
             _no_tyrant_msg_emitted = True
 
     if tyrant.pytyrant is None:
-        emit_no_tyrant_msg()
-        return None
+        return emit_no_tyrant_msg("not installed")
     try:
         tb = TyrantBackend()
         try:
             tb.open()
         except socket.error, exc:
             if exc.errno == errno.ECONNREFUSED:
-                emit_no_tyrant_msg()
-                return None
+                return emit_no_tyrant_msg("not running")
             else:
                 raise
         return tb
     except ImproperlyConfigured, exc:
-        return None
+        if "need to install" in str(exc):
+            return emit_no_tyrant_msg("not installed")
+        return emit_no_tyrant_msg("not configured")
 
 
 class TestTyrantBackend(unittest.TestCase):
@@ -63,13 +66,13 @@ class TestTyrantBackend(unittest.TestCase):
 
         tid = gen_unique_id()
 
-        self.assertFalse(tb.is_done(tid))
-        self.assertEquals(tb.get_status(tid), "PENDING")
+        self.assertFalse(tb.is_successful(tid))
+        self.assertEquals(tb.get_status(tid), states.PENDING)
         self.assertEquals(tb.get_result(tid), None)
 
         tb.mark_as_done(tid, 42)
-        self.assertTrue(tb.is_done(tid))
-        self.assertEquals(tb.get_status(tid), "DONE")
+        self.assertTrue(tb.is_successful(tid))
+        self.assertEquals(tb.get_status(tid), states.SUCCESS)
         self.assertEquals(tb.get_result(tid), 42)
         self.assertTrue(tb._cache.get(tid))
         self.assertTrue(tb.get_result(tid), 42)
@@ -98,8 +101,8 @@ class TestTyrantBackend(unittest.TestCase):
         except KeyError, exception:
             pass
         tb.mark_as_failure(tid3, exception)
-        self.assertFalse(tb.is_done(tid3))
-        self.assertEquals(tb.get_status(tid3), "FAILURE")
+        self.assertFalse(tb.is_successful(tid3))
+        self.assertEquals(tb.get_status(tid3), states.FAILURE)
         self.assertTrue(isinstance(tb.get_result(tid3), KeyError))
 
     def test_process_cleanup(self):
