@@ -1,6 +1,6 @@
 import unittest
 from django.conf import settings
-from celery.discovery import autodiscover
+from celery.loaders.djangoapp import autodiscover
 from celery.task import tasks
 
 
@@ -9,7 +9,6 @@ class TestDiscovery(unittest.TestCase):
     def assertDiscovery(self):
         apps = autodiscover()
         self.assertTrue(apps)
-        tasks.autodiscover()
         self.assertTrue("c.unittest.SomeAppTask" in tasks)
         self.assertEquals(tasks["c.unittest.SomeAppTask"].run(), 42)
 
@@ -21,4 +20,4 @@ class TestDiscovery(unittest.TestCase):
         if "someapp" in settings.INSTALLED_APPS:
             settings.INSTALLED_APPS = settings.INSTALLED_APPS + \
                     ["xxxnot.aexist"]
-            self.assertDiscovery()
+            self.assertRaises(ImportError, autodiscover)
